# TIPOS DE COMPROBANTE
INGRESO = 'I'
EGRESO = 'E'
PAGO = 'P'
TRASLADO = 'T'

# TIPOS DE COMPLEMENTO
COMPLEMENTO_DONATARIAS = 'Donatarias'
COMPLEMENTO_PAGOS = 'Pago'
COMPLEMENTO_CARTA_PORTE = 'CartaPorte'

# SUBTIPOS

ST_FACTURA = 2
ST_RECIBO_ARRENDAMIENTO = 3
ST_RECIBO_DONATIVO = 4